# --------------------------------------------------------------------------------------------------
# Copyright (c) Lukas Vik. All rights reserved.
#
# This file is part of the tsfpga project, a project platform for modern FPGA development.
# https://tsfpga.com
# https://github.com/tsfpga/tsfpga
# --------------------------------------------------------------------------------------------------

# This call is duplicated in tcl.py for synthesis
report_utilization -hierarchical -hierarchical_depth 4 -file "hierarchical_utilization.rpt"
